import QtQuick 1.0
import QtMobility.systeminfo 1.1
import "speedtestFunc.js" as SpeedtestFuncJS
import "history.js" as HistoryJS
import "config.js" as ConfigJS

Rectangle {
    id: speedTestView
    width: 480
    height: 820

    property variant mNetworkType
    property variant mDate
    property variant mDownload
    property variant mUpload
    property variant mPing

    property string mDownloadSpeed
    property string mUploadSpeed
    property string mPingTime

    property int mPingProgress: 0
    property int mDownloadProgress: 0
    property int mUploadProgress: 0

    property bool mPingBar: false
    property bool mDownloadBar: false
    property bool mUploadBar: false

    property bool mPingLoading: false
    property bool mDownloadLoading: false
    property bool mUploadLoading: false

    property string displayUnits: "kbps"

    property double speedFactor: 1.0

    property variant mPrevSpeed: 0
    property bool engineState: true
    property bool isRunning


    function delay() {
        cpnDial.value = 0;
        tmrDelay.running = true;
    }

    function formatTime() {
        var v = new Date();
        return [
            Qt.formatDateTime(v, "dd/MM/yy hh:mm:ss AP")
            //Qt.formatDateTime(v, "dd-MM-yy")
        ]
    }

    function initialize() {
        speedtestEngine.mDownloadSpeed = 0;
        speedtestEngine.mUploadSpeed = 0;
        speedtestEngine.mPingTime = 0;
        speedtestEngine.mPingProgress = 0;
        speedtestEngine.mDownloadProgress = 0;
        speedtestEngine.mUploadProgress = 0;
        speedtestEngine.mPingBar = true;
        speedtestEngine.mDownloadBar = false;
        speedtestEngine.mUploadBar = false;
        speedtestEngine.txtPingVisible = false;
        speedtestEngine.txtDownloadVisible = false;
        speedtestEngine.txtUploadVisible = false;
    }

    function setupTesting() {
        console.debug("setup");
        initialize();
        downloadSpeedList.clear();
        uploadSpeedList.clear();
        ConfigJS.loadConfigures();
        SpeedtestFuncJS.currentSpeed = 0;
        if ( configuresModel.count !== 0 ) {
            SpeedtestFuncJS.speedUnits = configuresModel.get(0).mDisplayUnits;
            speedtestEngine.displayUnits = SpeedtestFuncJS.speedUnits;
            speedtestEngine.animationSpeed = configuresModel.get(0).mAnimationSpeed;
        }
        //SpeedtestFuncJS.currentSpeed = 0;
        SpeedtestFuncJS.pingTest();
        //HistoryJS.clearAllHistory();
        if (networkInfo.mode == 1) {
            mNetworkType = "GSM";
            speedtestEngine.mNetworkType = "GSM";
        }
        else {
            mNetworkType = "WLAN";
            speedtestEngine.mNetworkType = "WLAN";
        }
        console.log(formatTime());
        mDate = "" + speedTestView.formatTime();
        //mDownload = "123kbps";
        //mUpload = "345kbps";
        //mPing = "12ms";
        //HistoryJS.saveHistory();

    }

    Component.onCompleted: {
        tabbar1.selectedButton = "Speedtest";
        console.log("pageDepth: " + appWindow.pageStack.depth)
        console.log("page:" + appWindow.pageStack.currentPage)
        speedtestEngine.txtPingLoading = true;
        speedTestView.mPingLoading = false;
        txtPingLoading.visible = speedTestView.mPingLoading;
        //txtDownloadSpeed.text = (speedtestEngine.mDownloadSpeed/speedtestEngine.speedFactor) + speedtestEngine.displayUnits;
        //txtUploadSpeed.text = (speedtestEngine.mUploadSpeed/speedtestEngine.speedFactor) + speedtestEngine.displayUnits;
        if (!speedtestEngine.isTested) {
            speedtestEngine.isTested = true;
            console.log("running: " + speedtestEngine.isRunning)
            if (!isRunning) setupTesting();
        }
    }

   Component.onDestruction: {
       console.log("destroy");
   }

   onVisibleChanged:  {
       console.log("visible");
       tabbar1.selectedButton = "Speedtest";

       if (speedtestEngine.displayUnits == 'kb/s')
           speedtestEngine.speedFactor = 8.0;
       else if (speedtestEngine.displayUnits == 'kbps')
           speedtestEngine.speedFactor = 1.0;
       else
           speedtestEngine.speedFactor = 1024.0;
   }

   Timer {
       id: tmrDelay
       interval: speedtestEngine.animationSpeed /*1000*/; running: false; repeat: false

       onTriggered: {
           tmrDelay.running = false;
           if (speedtestEngine.engineState == 3)  SpeedtestFuncJS.simpleUpload();
       }
   }


    Timer {
        id: tmrTrigger
        interval: 1100 - speedtestEngine.animationSpeed; running: speedtestEngine.isRunning; repeat: true
        onTriggered: {
            //console.debug("trig"+speedtestEngine.animationSpeed)
            if (speedtestEngine.engineState == 1) {
                cpnDial.value = Math.floor(speedtestEngine.mDownloadSpeed/ 12) ; //Math.floor(SpeedtestFuncJS.currentSpeed) * 15;
                downloadSpeedList.clear();
                uploadSpeedList.clear();
            }

            else if (speedtestEngine.engineState == 2) {
                //cpnDial.value = Math.floor(speedtestEngine.mDownloadSpeed/ 12) ; //Math.floor(SpeedtestFuncJS.currentSpeed) * 15;
                //cpnDial.value = Math.floor(speedtestEngine.mDownloadSpeed * 20.0 / 256.0);
                //speedtestEngine.mDownloadSpeed = 20000;
                if (speedtestEngine.mDownloadSpeed >= 21000) {
                    cpnDial.value = 102 + Math.floor(speedtestEngine.mDownloadSpeed * 25.0 / 30000.0);
                }
                else if (speedtestEngine.mDownloadSpeed >= 10000) {
                    cpnDial.value = 80 + Math.floor(speedtestEngine.mDownloadSpeed * 25.0 / 15000.0);
                }
                else if (speedtestEngine.mDownloadSpeed >= 1000) {
                    cpnDial.value = 67 + Math.floor(speedtestEngine.mDownloadSpeed * 25.0 / 10000.0);
                }
                else {
                    cpnDial.value = Math.floor(speedtestEngine.mDownloadSpeed * 20.0 / 256.0);
                }

                //console.log(speedTestView.mPrevSpeed + ", " + SpeedtestFuncJS.currentSpeed);
                if(downloadSpeedList.count > 10) {
                    downloadSpeedList.remove(0);
                }

                if (speedtestEngine.mDownloadSpeed > 10000) {
                    downloadSpeedList.append({'startSpeed': 53 - (speedTestView.mPrevSpeed), 'stopSpeed': 53 - (speedtestEngine.mDownloadSpeed/1000)});
                    speedTestView.mPrevSpeed = speedtestEngine.mDownloadSpeed/1000;
                }
                else if (speedtestEngine.mDownloadSpeed > 1000) {
                    downloadSpeedList.append({'startSpeed': 53 - (speedTestView.mPrevSpeed), 'stopSpeed': 53 - (speedtestEngine.mDownloadSpeed/100)});
                    speedTestView.mPrevSpeed = speedtestEngine.mDownloadSpeed/100;
                }
                else if (speedtestEngine.mDownloadSpeed > 100) {
                    downloadSpeedList.append({'startSpeed': 53 - (speedTestView.mPrevSpeed), 'stopSpeed': 53 - (speedtestEngine.mDownloadSpeed/50)});
                    speedTestView.mPrevSpeed = speedtestEngine.mDownloadSpeed/50;
                }
                else {
                    downloadSpeedList.append({'startSpeed': 53 - (speedTestView.mPrevSpeed), 'stopSpeed': 53 - (speedtestEngine.mDownloadSpeed/5)});
                    speedTestView.mPrevSpeed = speedtestEngine.mDownloadSpeed/5;
                }

                 speedtestEngine.mPrevSpeed = speedTestView.mPrevSpeed;
            }
            else if (speedtestEngine.engineState == 3){
                cpnDial.value = 0;
            }
            else if (speedtestEngine.engineState == 4){
                //cpnDial.value = Math.floor(speedtestEngine.mUploadSpeed/ 12) ; //Math.floor(SpeedtestFuncJS.currentSpeed) * 15;
                //cpnDial.value = Math.floor(speedtestEngine.mUploadSpeed * 20.0 / 256.0);
                if (speedtestEngine.mUploadSpeed >= 7000) {
                    cpnDial.value = 110 + Math.floor(speedtestEngine.mUploadSpeed * 25.0 / 10000.0);
                }
                else if (speedtestEngine.mUploadSpeed >= 3000) {
                    cpnDial.value = 69 + Math.floor(speedtestEngine.mUploadSpeed * 25.0 / 3000.0);
                }
                else if (speedtestEngine.mUploadSpeed >= 1000) {
                    cpnDial.value = 69 + Math.floor(speedtestEngine.mUploadSpeed * 25.0 / 5000.0);
                }
                else {
                    cpnDial.value = Math.floor(speedtestEngine.mUploadSpeed * 20.0 / 256.0);
                }

                if(uploadSpeedList.count > 10) {
                    uploadSpeedList.remove(0);
                }
                if (speedtestEngine.mUploadSpeed > 10000) {
                    uploadSpeedList.append({'startSpeed': 53 - (speedTestView.mPrevSpeed), 'stopSpeed': 53 - (speedtestEngine.mUploadSpeed/1000)});
                    speedTestView.mPrevSpeed = speedtestEngine.mUploadSpeed/1000;
                }
                else if (speedtestEngine.mUploadSpeed > 1000) {
                    uploadSpeedList.append({'startSpeed': 53 - (speedTestView.mPrevSpeed), 'stopSpeed': 53 - (speedtestEngine.mUploadSpeed/100)});
                    speedTestView.mPrevSpeed = speedtestEngine.mUploadSpeed/100;
                }
                else if (speedtestEngine.mUploadSpeed > 100) {
                    uploadSpeedList.append({'startSpeed': 53 - (speedTestView.mPrevSpeed), 'stopSpeed': 53 - (speedtestEngine.mUploadSpeed/50)});
                    speedTestView.mPrevSpeed = speedtestEngine.mUploadSpeed/50;
                }
                else {
                    uploadSpeedList.append({'startSpeed': 53 - (speedTestView.mPrevSpeed), 'stopSpeed': 53 - (speedtestEngine.mUploadSpeed/5)});
                    speedTestView.mPrevSpeed = speedtestEngine.mUploadSpeed/5;
                }

                 speedtestEngine.mPrevSpeed = speedTestView.mPrevSpeed;

            }
        }
    }

    Image {
        id: image1
        x: 0
        y: 0
        width: 480
        height: 854
        source: "images/bg.png"

        Dial {
            id: cpnDial
            x: 0
            y: 313
            value: 0
        }

        ResultBar {
            id: resultbar1
            x: 0
            y: 72

            ProgressBarItem {
                id: pbPing
                x: 25
                y: 70
                visible: speedtestEngine.mPingBar
                value: speedtestEngine.mPingProgress
            }

            ProgressBarItem {
                id: pbDownload
                x: 176
                y: 70
                visible: speedtestEngine.mDownloadBar
                value: speedtestEngine.mDownloadProgress
            }

            ProgressBarItem {
                id: pbUpload
                x: 338
                y: 70
                visible: speedtestEngine.mUploadBar
                value: speedtestEngine.mUploadProgress
            }
        }

        RestartBar {
            id: restartbar1
            x: 0
            y: 229
            visible: !speedtestEngine.isRunning

            onVisibleChanged: { console.log("restart"); cpnDial.value = 0; speedtestEngine.mCurrentSpeed = 0; speedTestView.mPrevSpeed = 0.0; }

            onClicked: {
                speedtestEngine.isRunning = false;
                speedtestEngine.connectionError = false;
                setupTesting();
            }

        }


        TitleBar {
            id: titlebar1
            x: 0
            y: 12
            width: 480
            anchors.horizontalCenter: parent.horizontalCenter

            QuitButton {
                id: cpnQuitBtn
                x: 329
                y: -6
                anchors.right: parent.right
                anchors.rightMargin: 11
                anchors.verticalCenter: parent.verticalCenter
            }
        }


    }

    Text {
        id: txtDownloadLoading
        x: 188
        y: 146
        text: qsTr("Loading...")
        font.pixelSize: 18
        visible: speedtestEngine.txtDownloadLoading
    }

    Text {
        id: txtDownloadSpeed
        x: 166
        y: 162
        width: 99
        height: 24
        text: (speedtestEngine.mDownloadSpeed/speedtestEngine.speedFactor).toFixed(2) + speedtestEngine.displayUnits
        visible: speedtestEngine.txtDownloadVisible
        verticalAlignment: Text.AlignVCenter
        horizontalAlignment: Text.AlignHCenter
        font.pixelSize: 20
    }

    Text {
        id: txtUploadLoading
        x: 348
        y: 146
        text: qsTr("Loading...")
        font.pixelSize: 18
        visible: speedtestEngine.txtUploadLoading
    }

    Text {
        id: txtUploadSpeed
        x: 348
        y: 162
        width: 70
        height: 24
        text: (speedtestEngine.mUploadSpeed/speedtestEngine.speedFactor).toFixed(2) + speedtestEngine.displayUnits
        visible: speedtestEngine.txtUploadVisible
        verticalAlignment: Text.AlignVCenter
        horizontalAlignment: Text.AlignHCenter
        font.pixelSize: 20
    }

    Text {
        id: txtPingLoading
        x: 34
        y: 146
        text: qsTr("Loading...")
        font.pixelSize: 18
        visible: speedTestView.mPingLoading
    }

    Text {
        id: txtPingTime
        x: 22
        y: 162
        width: 99
        height: 24
        text: speedtestEngine.mPingTime
        visible: speedtestEngine.txtPingVisible
        font.pixelSize: 20
        horizontalAlignment: Text.AlignHCenter
        verticalAlignment: Text.AlignVCenter
    }

    CurrentBar {
        id: currentbar1
        x: 0
        y: 571
        //anchors.bottom: tabbar1.top

        Text {
            id: text1
            x: 196
            y: 50
            width: 88
            height: 33
            text: "" + (speedtestEngine.mCurrentSpeed/speedtestEngine.speedFactor).toFixed(2)
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter

            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 27
        }
    }

    TabBar {
        id: tabbar1
        x: 0
        y: 550
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 0
        selectedButton: "Speedtest"

        onTabButtonClicked: {
            console.log(targetState);
            //appWindow.state = targetState;
            switch (targetState) {
            case "SpeedtestView":
                {
                 break;
                }

            case "SettingView":
                {
                 pageStack.pop();
                 pageStack.push(Qt.resolvedUrl("SettingView.qml"));
                 break;
                }

            case "ResultView" :
                {
                 pageStack.pop();
                 pageStack.push(Qt.resolvedUrl("ResultView.qml"));
                 break;
                }

            case "AboutView":
                {
                 pageStack.pop();
                 pageStack.push(Qt.resolvedUrl("AboutView.qml"));
                 break;
                }

            default: break;
            }
        }
    }

    Rectangle {
        id: rectangle1
        x: 33
        y: 632
        width: 120
        height: 53
        color: "#00000000"

        GridView {
            id: graphDownload
            anchors.fill: parent
            cellWidth: 10
            cellHeight: 53
            delegate: Component {
                LinePlot {
                    posX1: 0
                    posY1: startSpeed
                    posX2: graphDownload.cellWidth
                    posY2: stopSpeed
                }
            }
            model: downloadSpeedList
        }
    }

    Rectangle {
        id: rectangle2
        x: 334
        y: 632
        width: 120
        height: 53
        color: "#00000000"

        GridView {
            id: graphUpload
            anchors.fill: parent
            cellWidth: 8
            cellHeight: 53
            delegate: Component {
                LinePlot {
                    posX1: 0
                    posY1: startSpeed
                    posX2: graphUpload.cellWidth
                    posY2: stopSpeed
                }
            }
            model: uploadSpeedList
        }
    }


    ListModel {
        id: configuresModel

        ListElement {
            mName: "conf"
            mDisplayUnits: "kbps"
            mSortANme: "mDate"
            mAnimationSpeed: "500"
        }
    }


    NetworkInfo {
        id: networkInfo
    }

    ErrorDialogItem {
        id: errordialogitem1
        opacity: 0
        visible: true
        anchors.fill: parent
        state: (speedtestEngine.connectionError) ? "show" : ""
        onAccepted: {
            speedtestEngine.isRunning = false;
            speedtestEngine.connectionError = false;
            errordialogitem1.close();
        }
    }
}
